/* -*- c++ -*-
 * tkresolver - Telekinesis testing software
 *
 * Copyright (C) 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdexcept>

#include "editor/item_tree.h"
#include "editor/domain.h"
#include "editor/experiment_store.h"
#include "editor/location_store.h"
#include "editor/operator_store.h"

using namespace Tk::Editor;

ItemTree:: ItemTree(Tk::Db::Database *database, Domain &domain, Constraint &constraint)
  : _database(database)
    //    _column("A column", _renderer)
{
  WDEBUG_OUT << L"initialising ItemTree" << std::endl;

  set_headers_visible(false);
  _column.pack_start(_renderer);
  append_column(_column);
  
  add_store("experiment", new ExperimentStore(database));
  add_store("location", new LocationStore(database));
  add_store("operator", new OperatorStore(database));
  
  set_domain(domain.get_domain());
  
  domain.signal_domain_changed().connect( sigc::mem_fun(*this, &ItemTree::set_domain) );
  constraint.signal_constraint_activated().connect( sigc::mem_fun(*this, &ItemTree::set_entities) );
  get_selection()->signal_changed().connect( sigc::mem_fun(*this, &ItemTree::selection_changed) );
  
  WDEBUG_OUT << L"initialised ItemTree" << std::endl;
}

void
ItemTree:: add_store(const Tk::String &domain, ItemStore *item_store)
{
  item_store->refresh();
  Glib::RefPtr<ItemStore> item_store_ptr(item_store);
  _stores.insert(DomainItemStores::value_type(domain, item_store_ptr));
}

void
ItemTree:: connect_create_button(Gtk::Button &create_button)
{
  create_button.signal_clicked().connect( sigc::mem_fun(*this, &ItemTree::create) );
}


void
ItemTree:: set_domain(const Tk::String &domain)
{
  DomainItemStores::iterator iter( _stores.find(domain) );
  
  if (iter == _stores.end())
    throw std::runtime_error(std::string("Could not find item store for domain `") + static_cast<std::string>(domain) + "'");
  
  _current = iter->second;
  
  set_model(_current);
  
  _column.set_cell_data_func(_renderer, _current->get_cell_data_slot());
}

void
ItemTree:: set_entities(const Tk::Db::Ids &ids)
{
  _current->set_entities(ids);
}

ItemTree::SignalEntitySelected
ItemTree:: signal_entity_selected()
{
  return _signal_entity_selected;
}


void
ItemTree:: create()
{
  Tk::String domain( _current->get_domain() );
  Tk::Db::Id entity_id( _database->create_entity(domain) );
  
  _current->refresh();
  
  Gtk::TreePath path(_current->find_entity_row(entity_id));
  set_cursor(path);
}

Tk::Db::Id
ItemTree:: get_selected_entity()
{
  return _current->get_row_entity(get_selection()->get_selected());
}

void
ItemTree:: selection_changed()
{
  if (!get_selection()->get_selected())
    return;
  
  _signal_entity_selected.emit(_current->get_domain(), get_selected_entity());
}
